const { Clutter, GLib, Shell, St } = imports.gi;

const Main = imports.ui.main;
const ExtensionUtils = imports.misc.extensionUtils;

const { WindowSwitcherPopup } = imports.ui.altTab;
const OverviewControlsState = imports.ui.overviewControls.ControlsState;
const {TouchpadSwipeGesture} = ExtensionUtils.getCurrentExtension().imports.src.swipeTracker;

const AltTabExtState = {
    NONE: 0,
    ALTTAB: 1,
    OVERVIEWTAB: 2
};

var AltTabGestureExtension = class AltTabGestureExtension {
    constructor() {
        this._connectHandlers = [];
        this._touchpadSwipeTracker = null;
        this._adjustment = null;
        this._switcher = null;
        this._workspaceContainerLayout = null;
        this._windowPreview = null;
        this._extState = AltTabExtState.NONE;
    }

    enable() {
        this._touchpadSwipeTracker = new TouchpadSwipeGesture(
            [3],
            Shell.ActionMode.ALL,
            Clutter.Orientation.HORIZONTAL,
            false);

        this._adjustment = new St.Adjustment({
            value: 0,
            lower: 0,
            upper: 1,
        });
        this._adjustment.connect('notify::value', this._onUpdateAdjustmentValue.bind(this));

        this._connectHandlers.push(this._touchpadSwipeTracker.connect('begin', this._gestureBegin.bind(this)));
        this._connectHandlers.push(this._touchpadSwipeTracker.connect('update', this._gestureUpdate.bind(this)));
        this._connectHandlers.push(this._touchpadSwipeTracker.connect('end', this._gestureEnd.bind(this)));
    }

    disable() {
        this._connectHandlers.forEach(handle => this._touchpadSwipeTracker.disconnect(handle));
        this._connectHandlers = [];
        if (this._touchpadSwipeTracker) {
            delete this._touchpadSwipeTracker;
        }
        if (this._switcher) {
            delete this._switcher;
        }
        if (this._adjustment) {
            delete this._adjustment;
        }
    }

    _onUpdateAdjustmentValue() {
        // log(`adjustment: value = ${this._adjustment.value}`);
        if (this._extState === AltTabExtState.ALTTAB) {
            let nelement = this._switcher._items.length;
            // log(`adjustment: alt-tab nelement = ${nelement}`)
            if (nelement > 1) {
                let n = Math.floor((nelement+2)  * this._adjustment.value);
                n = Math.clamp(n, 1, nelement) - 1;
                // log(`adjustment: n = ${n}`);
                this._switcher._select(n);
            }
        }
        // else if (this._extState === AltTabExtState.OVERVIEWTAB) {
        //     let nelement = this._workspaceContainerLayout._sortedWindows.length;
        //     // log(`adjustment: overview-tab nelement = ${nelement}`)
        //     if (nelement > 0) {
        //         let n = Math.round((nelement+2) * this._adjustment.value);
        //         n = Math.clamp(n, 1, nelement) - 1;
        //         // log(`adjustment: n = ${n}`);
        //         this._windowPreview = this._workspaceContainerLayout._sortedWindows[n];
        //         this._windowPreview.showOverlay(false);
        //     }
        // }
    }

    _gestureBegin(gesture, time, x, y) {
        // log(`ext: gesture begin, time=${time}`);
        this._progress = 0;
        if (Main.actionMode === Shell.ActionMode.NORMAL && 
            this._extState === AltTabExtState.NONE
            ) {
            if (this._switcher) {
                this._switcher.destroy();
            }
            this._switcher = new WindowSwitcherPopup();
            let nelement = this._switcher._items.length
            if (nelement > 0) {
                this._switcher.show(false, "switch-windows", 0);
                if (this._switcher._noModsTimeoutId != 0) {
                    GLib.source_remove(this._switcher._noModsTimeoutId);
                    this._switcher._noModsTimeoutId = 0;
                }
                if (nelement === 1) {
                    this._switcher._select(0);
                    this._progress = 0;
                } else {
                    this._progress = 2.5 / (nelement+2);
                    this._switcher._select(1);
                }
                this._switcher._showImmediately();
                this._extState = AltTabExtState.ALTTAB;
            } else {
                this._switcher.destroy();
            }
        }
        // else if (Main.actionMode === Shell.ActionMode.OVERVIEW && 
        //     Main.overview._overview.controls._stateAdjustment.value === OverviewControlsState.WINDOW_PICKER &&
        //     this._extState === AltTabExtState.NONE
        //     ) {
        //     let monitorIndex = Main.layoutManager.currentMonitor.index;
        //     let workspaceDisplay = Main.overview._overview.controls._workspacesDisplay;
        //     if (workspaceDisplay._workspacesViews.length > monitorIndex) {
        //         let workspace = workspaceDisplay._workspacesViews[monitorIndex].getActiveWorkspace();
        //         this._workspaceContainerLayout = workspace._container.layout_manager;
        //         if (this._workspaceContainerLayout._sortedWindows.length > 0) {
        //             this._windowPreview = this._workspaceContainerLayout._sortedWindows[0];
        //             this._windowPreview.showOverlay();
        //             this._extState = AltTabExtState.OVERVIEWTAB;
        //         }
        //     }
        // }
    }

    _gestureUpdate(gesture, time, delta, distance) {
        if (this._extState !== AltTabExtState.NONE) {
            this._progress = Math.clamp(this._progress + delta / distance, 0, 1);
            // log(`ext: gesture update, time=${time}, distance=${distance}, delta=${delta}, progress=${this._progress}`);
            this._adjustment.value = this._progress;
        }
    }

    _gestureEnd(gesture, time, distance) {
        // log(`ext: gesture end, time=${time}, distance=${distance}`);
        if (this._extState === AltTabExtState.ALTTAB) {
            this._extState = AltTabExtState.NONE;
            let win = this._switcher._items[this._switcher._selectedIndex].window;
            // this._switcher.remove_actor(this._switcher._switcherList);
            // this._switcher._switcherList.destroy();
            this._switcher.destroy();
            this._switcher = null;
            Main.activateWindow(win);
            this._progress = 0;
            this._adjustment.value = 0;
            // Main.overview.hide();
        } 
        // else if (this._extState === AltTabExtState.OVERVIEWTAB) {
        //     this._extState = AltTabExtState.NONE;
        //     if (this._windowPreview) {
        //         this._windowPreview._activate();
        //         this._windowPreview = null;
        //     }
        //     this._workspaceContainerLayout = null;
        // }
        
        // log(`ext: gesture ended`);
    }
};