const { GObject } = imports.gi;
const Main = imports.ui.main;
const ExtensionUtils = imports.misc.extensionUtils;

const { TouchpadSwipeGesture } = ExtensionUtils.getCurrentExtension().imports.src.swipeTracker;

var GestureExtension = class GestureExtension {
    constructor() {
        this._swipeTrackers = [
            /// swipeTracker, number of fingers for gesture, old number of fingers
            [Main.overview._swipeTracker, [3, 4], [3]],
            // [Main.overview._overview.controls._workspacesDisplay._swipeTracker, [3,4], [3]],
            [Main.wm._workspaceAnimation._swipeTracker, [4], [3]]
        ];

        this._workspacesDisplay = Main.overview._overview.controls._workspacesDisplay;
    }

    enable() {
        this._swipeTrackers.forEach(entry => {
            let [swipeTracker, nfingers, _] = entry;
            let touchpadGesture = new TouchpadSwipeGesture(
                nfingers,
                swipeTracker._allowedModes,
                swipeTracker.orientation,
                true);

            this._attachGestureToTracker(swipeTracker, touchpadGesture);
        });
        let wsDisplayOldTouchpadGesture = this._workspacesDisplay._swipeTracker._touchpadGesture;
        this._workspacesDisplay._swipeTracker._touchpadGesture = null;
        this._attachGestureToTracker(
            this._workspacesDisplay._swipeTracker,
            new TouchpadSwipeGesture(
                [4],
                this._workspacesDisplay._swipeTracker._allowedModes,
                this._workspacesDisplay._swipeTracker.orientation,
                true)
        );
        this._wsDisplayNewTouchpadGesture = this._workspacesDisplay._swipeTracker._touchpadGesture;
        this._workspacesDisplay._swipeTracker._touchpadGesture = wsDisplayOldTouchpadGesture;
    }

    disable() {
        this._swipeTrackers.forEach(entry => {
            let [swipeTracker, _, nfingers] = entry;
            let touchpadGesture = new TouchpadSwipeGesture(
                nfingers,
                swipeTracker._allowedModes,
                swipeTracker.orientation,
                true);
            this._attachGestureToTracker(swipeTracker, touchpadGesture);
        });
        if (this._wsDisplayNewTouchpadGesture) {
            this._wsDisplayNewTouchpadGesture.destroy();
        }
    }

    _attachGestureToTracker(swipeTracker, touchpadSwipeGesture) {
        if (swipeTracker._touchpadGesture) {
            swipeTracker._touchpadGesture.destroy();
        }
        swipeTracker._touchpadGesture = touchpadSwipeGesture;
        swipeTracker._touchpadGesture.connect('begin', swipeTracker._beginGesture.bind(swipeTracker));
        swipeTracker._touchpadGesture.connect('update', swipeTracker._updateGesture.bind(swipeTracker));
        swipeTracker._touchpadGesture.connect('end', swipeTracker._endTouchpadGesture.bind(swipeTracker));
        swipeTracker.bind_property('enabled', swipeTracker._touchpadGesture, 'enabled', 0);
        swipeTracker.bind_property('orientation', swipeTracker._touchpadGesture, 'orientation',
            GObject.BindingFlags.SYNC_CREATE);
    }
};